/*
 * This software is distributed under following license based on modified BSD
 * style license.
 * ----------------------------------------------------------------------
 * 
 * Copyright 2009 The Nimbus2 Project. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE NIMBUS PROJECT ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
 * NO EVENT SHALL THE NIMBUS PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of the Nimbus2 Project.
 */
package jp.ossc.nimbus.service.aop.interceptor;

import java.util.Random;

import jp.ossc.nimbus.core.*;
import jp.ossc.nimbus.service.aop.*;

/**
 * SleepC^[Zv^B<p>
 * \bȟĂяoɑ΂āA莞ԂsleepC^[Zv^łB<br>
 * ȉɁASleepC^[Zv^̃T[rX`B<br>
 * <pre>
 * &lt;?xml version="1.0" encoding="Shift_JIS"?&gt;
 * 
 * &lt;nimbus&gt;
 *     
 *     &lt;manager name="Sample"&gt;
 *         
 *         &lt;service name="SleepInterceptor"
 *                  code="jp.ossc.nimbus.service.aop.interceptor.SleepInterceptorService"&gt;
 *             &lt;attribute name="SleepTime"&gt;500&lt;/attribute&gt;
 *         &lt;/service&gt;
 *         
 *     &lt;/manager&gt;
 *     
 * &lt;/nimbus&gt;
 * </pre>
 *
 * @author M.Takata
 */
public class SleepInterceptorService extends ServiceBase
 implements Interceptor, SleepInterceptorServiceMBean{
    
    private static final long serialVersionUID = 4049992073980961148L;
    
    private long sleepTime;
    private int randomSleepTime;
    
    // SleepInterceptorServiceMBeanJavaDoc
    public void setSleepTime(long time){
        sleepTime = time;
    }
    // SleepInterceptorServiceMBeanJavaDoc
    public long getSleepTime(){
        return sleepTime;
    }
    
    // SleepInterceptorServiceMBeanJavaDoc
    public void setRandomSleepTime(int time){
        randomSleepTime = time;
    }
    // SleepInterceptorServiceMBeanJavaDoc
    public int getRandomSleepTime(){
        return randomSleepTime;
    }
    
    /**
     * w肳ꂽԂsleepāÃC^[Zv^ĂяoB<p>
     * T[rXJnĂȂꍇ́AɎ̃C^[Zv^ĂяoB<br>
     *
     * @param context ĂяõReLXg
     * @param chain ̃C^[Zv^Ăяo߂̃`F[
     * @return Ăяoʂ̖߂l
     * @exception Throwable ĂяoŗOꍇA܂͂̃C^[Zv^ŔCӂ̗OꍇBAA{Ăяo鏈throwȂRuntimeExceptionȊO̗OthrowĂAĂяoɂ͓`dȂB
     */
    public Object invoke(
        InvocationContext context,
        InterceptorChain chain
    ) throws Throwable{
        if(getState() == State.STARTED){
            
            long currentSleepTime = 0;
            if(sleepTime > 0){
                currentSleepTime = sleepTime;
            }else if(randomSleepTime > 0){
                Random random = new Random(System.currentTimeMillis());
                currentSleepTime = random.nextInt(randomSleepTime);
            }
            if(currentSleepTime > 0){
                try{
                    Thread.sleep(currentSleepTime);
                }catch(InterruptedException e){
                    Thread.interrupted();
                }
            }
        }
        return chain.invokeNext(context);
    }
}
