/*
 * This software is distributed under following license based on modified BSD
 * style license.
 * ----------------------------------------------------------------------
 * 
 * Copyright 2011 The Nimbus2 Project. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE NIMBUS PROJECT ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
 * NO EVENT SHALL THE NIMBUS PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of the Nimbus2 Project.
 */
package jp.ossc.nimbus.beans;

import static org.testng.Assert.fail;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import static org.testng.Assert.assertEquals;

import org.testng.annotations.Test;
import org.testng.annotations.DataProvider;

/**
 * {@link DateEditor}eXgP[XB<p>
 * 
 * @author Y.Kunisaki
 */
public class DateEditorTest extends BeansTestBase{
    @DataProvider(name = "SetAsText")
    public Object[][] dataSetAsText(){
        try{
            return new Object[][]{
                    {"2008/07/29 15:49:11 100", new SimpleDateFormat("yyyy/MM/dd HH:mm:ss SSS").parse("2008/07/29 15:49:11 100")},
                    {"//15 15::11", new SimpleDateFormat("yyyy/MM/dd HH:mm:ss SSS").parse("1970/01/15 15:00:11 000")}
            };
        } catch (ParseException e) {
            fail(e.getMessage());
        }
        
        return null;
    }
    @Test(groups = {"jp.ossc.nimbus.beans.DateEditor#setAsText(java.lang.String)",
                    "jp.ossc.nimbus.beans.DateEditor#getValue()",
                    "Normal"},
          dataProvider = "SetAsText")
    public void testSetText(String paramStr, Date answer){
        editorSetText(DateEditor.class, paramStr, answer);
    }
    
    @Test(groups = {"jp.ossc.nimbus.beans.DateEditor#setAsValue(java.lang.Character)",
                    "jp.ossc.nimbus.beans.DateEditor#getAsText()",
                    "Normal"})
    public void testSetValue(){
        DateEditor editor = new DateEditor();
        Date date = new Date();
        editor.setValue(date);
        assertEquals(
            new SimpleDateFormat("yyyy/MM/dd HH:mm:ss SSS").format(date),
            editor.getAsText()
        );
    }
}
